import { z } from "https://deno.land/x/zod@v3.22.4/mod.ts";

/**
 * Wraps the CRI API.
 */
export class CriApi {
	private token;

	public constructor(token: string) {
		this.token = token;
	}

	/**
	 * Fetches an array of the groups an user has been part of.
	 */
	public async groups_of(login: string) {
		const response = await fetch(`https://cri.epita.fr/api/v2/users/${login}/`, {
			headers: {
				accept: "application/json",
				authorization: "Basic " + this.token,
			},
		});
		const body = await response.json();
		const group = z.object({ slug: z.string() });
		const parser = z.object({
			primary_group: group,
			groups_history: z.array(z.object({
				group,
				graduation_year: z.number().or(z.null()),
			})),
			current_groups: z.array(group),
		});
		const parsed = parser.parse(body);
		const result = new Set<string>();
		result.add(parsed.primary_group.slug);
		for (const group of parsed.current_groups) result.add(group.slug);
		for (const { group } of parsed.groups_history) result.add(group.slug);
		return Array.from(result.values());
	}

	/**
	 * Tests wether a given login exists within the CRI registry.
	 */
	public async user_exists(login: string) {
		const response = await fetch(`https://cri.epita.fr/api/v2/users/${login}/`, {
			headers: {
				accept: "application/json",
				authorization: "Basic " + this.token,
			},
		});
		return response.status === 200;
	}
}
